/*
 * accelerometer.c
 *
 *  Created on: Feb 20, 2013
 *      Author: ed.nash@timesys.com
 *
 *  CGI for webgl demo - when requested, polls
 *
 *  TODO implement persistence with http server (not sure boa supports it)
 *
 *  TODO implement the following commands (right now the cmd field is ignored
 *  and treated as a sample request)
 *
 *  Commands from Linux / responses by MQX
 *
 *  SENSOR_ACCEL_INITIALIZE	/ MQX_OK or -1 error
 *  SENSOR_ACCEL_CALIBRATE	/ MQX_OK or -1 error
 *  SENSOR_ACCEL_SAMPLE		/ MQX_OK with X,Y,Z data or -1 error
 *
 */
#include <stdio.h>
#include <string.h>
#include <linux/mcc_config.h>
#include <linux/mcc_common.h>
#include <mcc_api.h>

// MCC constants
#define MQX_NODE 0
#define MQX_PORT 1
#define M4_CORE 1

#define THIS_NODE 0
#define THIS_PORT 1

static MCC_ENDPOINT m4ep = { M4_CORE, MQX_NODE, MQX_PORT };
static MCC_ENDPOINT a5ep; // filled in when created

struct mcc_msg_struct {
	int cmd;
	int status;
	int sequence;
	int x;
	int y;
	int z;
} packed;

int main(int argc, char** argv) {

	int ret_value, rec_len;
	MCC_INFO_STRUCT mcc_info;
	struct mcc_msg_struct msg;

	// cgi response needs to start with this
	printf("Content-type: text/html\n\n");

	// initialize the sub-system
	ret_value = mcc_initialize(THIS_NODE);
	if (ret_value != MCC_SUCCESS) {
		printf("Error: mcc_initialize error: %d for node: %d\n", ret_value, THIS_NODE);
		return ret_value;
	}

	// verify compatible versions
	ret_value = mcc_get_info(THIS_NODE, &mcc_info);
	if (ret_value != MCC_SUCCESS) {
		printf("Error: mcc_get_info error: %d for node: %d\n", ret_value, THIS_NODE);
		return ret_value;
	}

	if (strcmp(mcc_info.version_string, MCC_VERSION_STRING) != 0) {
		printf("Error: different versions of the MCC library on each core!\n");
		mcc_destroy(THIS_NODE);
		return MCC_ERR_DEV;
	}

	// create this side's mailbox
	ret_value = mcc_create_endpoint(&a5ep, THIS_PORT);
	if (ret_value != MCC_SUCCESS) {
		printf("Error: mcc_create_endpoint error: %d for node,port: %d, %d\n", 	ret_value, THIS_NODE, THIS_PORT);
		mcc_destroy(THIS_NODE);
		return ret_value;
	}

	ret_value = mcc_send(&m4ep, &msg, sizeof(msg), 500000);
	if (MCC_SUCCESS != ret_value) {
		printf("Error: mcc_send error: %d\n", ret_value);
		mcc_destroy(THIS_NODE);
		return 1;
	}

	ret_value = mcc_recv_copy(&a5ep, (char *) &msg, sizeof(msg), (MCC_MEM_SIZE *) &rec_len, 0xffffffff);
	if (MCC_SUCCESS != ret_value) {
		printf("Error: mcc_recv_copy error: %d\n", ret_value);
		mcc_destroy(THIS_NODE);
		return 1;
	}

	mcc_destroy(THIS_NODE);

	printf("%d,%d,%d\n", msg.x, msg.y, msg.z);

	return 0;
}
